! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! boundary module
module boundary_mod

  use sep

  implicit none
  integer, private   :: nx, nz, iz, ix,abc
  real, private      :: c1,c2
  real, private, allocatable, dimension(:) :: filter

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine boundary_init(nz_in,nx_in,abc_in)
    integer        :: nx_in, nz_in, abc_in, i
    real           :: aux

    nx = nx_in
    nz = nz_in
    abc= abc_in

    allocate(filter(abc))

    do i=1,abc
      aux = (i-1)/(0.15*float(abc))
      filter(i) = 1.0 - exp(-aux)
    enddo
    call sep_write(filter,"filter")

  endsubroutine

  ! Function that is called by elastic_mod
  ! These subroutines apply an absorbing boundary condition
  function boundary_op(adj,add,model,data) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data
    integer                     :: stat

    call boundary_op_top(adj,add,model,data)
    call boundary_op_left(adj,add,model,data)
    call boundary_op_right(adj,add,model,data)
    call boundary_op_bottom(adj,add,model,data)
    stat=0
  endfunction

  ! Absorbing boundary condition fo TOP border
  subroutine boundary_op_top(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    model=data
    do ix=1,nx
      do iz=1,abc
        model(iz,ix,1) = filter(iz)*data(iz,ix,1)
      enddo
    enddo
  endsubroutine

  ! Absorbing boundary condition fo LEFT border
  subroutine boundary_op_left(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    model=data
    do ix=1,abc
      do iz=1,nz
        model(iz,ix,1) = filter(ix)*data(iz,ix,1)
      enddo
    enddo
  endsubroutine

  ! Absorbing boundary condition fo RIGHT border
  subroutine boundary_op_right(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    model=data
    do ix=1,abc
      do iz=1,nz
        model(iz,nx-ix+1,1) = filter(ix)*data(iz,nx-ix+1,1)
      enddo
    enddo
  endsubroutine

  ! Absorbing boundary condition fo BOTTOM border
  subroutine boundary_op_bottom(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    model=data
    do ix=1,nx
      do iz=1,abc
        model(nz-iz+1,ix,1) = filter(iz)*data(nz-iz+1,ix,1)
      enddo
    enddo
  endsubroutine

endmodule
