! Main program to create 2D models
program create_models

  use sep

  implicit none

  integer                               :: nz,nx, iz, ix, thick, z1
  real                                  :: dz,oz,dx,ox
  real                                  :: vp1, vp2, rho2, vz, rho1
  real, allocatable, dimension(:,:)     :: vp, vs, rho

  call sep_init()

  ! Model parameters
  call from_param("nz",nz)
  call from_param("oz",oz)
  call from_param("dz",dz)
  call from_param("nx",nx)
  call from_param("ox",ox)
  call from_param("dx",dx)
  call from_param("vp1",vp1)
  call from_param("rho1",rho1)
  call from_param("vp2",vp2)
  call from_param("rho2",rho2)
  call from_param("thick",thick)
  call from_param("vz",vz,0.)
  call from_param("z1",z1)

  ! Write SEP header parameter
  call to_history("n1",nz)
  call to_history("o1",oz)
  call to_history("d1",dz)
  call to_history("n2",nx)
  call to_history("o2",ox)
  call to_history("d2",dx)
  call to_history("label1","z")
  call to_history("label2","x")
  call to_history("n1",nz,"rho")
  call to_history("o1",oz,"rho")
  call to_history("d1",dz,"rho")
  call to_history("n2",nx,"rho")
  call to_history("o2",ox,"rho")
  call to_history("d2",dx,"rho")
  call to_history("label1","z","rho")
  call to_history("label2","x","rho")
  call to_history("n1",nz,"vs")
  call to_history("o1",oz,"vs")
  call to_history("d1",dz,"vs")
  call to_history("n2",nx,"vs")
  call to_history("o2",ox,"vs")
  call to_history("d2",dx,"vs")
  call to_history("label1","z","vs")
  call to_history("label2","x","vs")

  ! Allocation of the input/output matrices
  allocate(vp(nz,nx))
  allocate(vs(nz,nx))
  allocate(rho(nz,nx))

  ! Initialize values
  vp=0.
  vs=0.
  rho=0.

  ! Create backgroun velocity
  do ix=1,nx
    do iz=1,nz
      vp(iz,ix)=vp1+vz*iz
      vs(iz,ix)=0.50*vp(iz,ix)
      rho(iz,ix)=rho1
    enddo
  enddo

  ! Insert fluid saturated horizontal layer
  do ix=1,nx
    do iz=z1,z1+thick
      vp(iz,ix)=vp2
      vs(iz,ix)=0.50*vs(iz,ix)
      rho(iz,ix)=rho2
    enddo
  enddo

  ! Write outputs
  call sep_write(vp)
  call sep_write(vs,"vs")
  call sep_write(rho,"rho")

  call sep_close

end program
