! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! finite difference module
module elasticlib_mod

  use sep

  implicit none
  integer, private   :: nx, nz, sp, iz, ix
  real, private      :: c1,c2,c3,c4,c5

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine elasticmod_init(nz_in,nx_in, sp_in)
    integer        :: nx_in, nz_in, sp_in

    nx = nx_in
    nz = nz_in
    sp = sp_in

  endsubroutine

  ! Functions that are called by elastic_mod

  ! These x derivatives are centered at x=+1/2 
  function dx_op(adj,add,model,data) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data
    integer                     :: stat

    if (sp.eq.4) then
      call dx4th_op2(adj,add,model,data)
    else
      call dx10th_op2(adj,add,model,data)
    endif
    stat=0
  endfunction

  ! These x derivatives are centered at x=-1/2 
  function dx_op3(adj,add,model,data) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data
    integer                     :: stat

    if (sp.eq.4) then
      call dx4th_op4(adj,add,model,data)
    else
      call dx10th_op4(adj,add,model,data)
    endif
    stat=0
  endfunction

  ! These z derivatives are centered at z=-1/2 
  function dz_op(adj,add,model,data) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data
    integer                     :: stat

    if (sp.eq.4) then
      call dz4th_op2(adj,add,model,data)
    else
      call dz10th_op2(adj,add,model,data)
    endif
    stat=0
  endfunction

  ! These z derivatives are centered at z=+1/2 
  function dz_op3(adj,add,model,data) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data
    integer                     :: stat

    if (sp.eq.4) then
      call dz4th_op4(adj,add,model,data)
    else
      call dz10th_op4(adj,add,model,data)
    endif
    stat=0
  endfunction

  ! Derivative used on the shear stress
  function dxz_op(adj,add,model,data1,data2) result(stat)
    logical,intent(in),optional :: adj,add
    real,dimension(:,:,:)       :: model,data1,data2
    integer                     :: stat

    if (sp.eq.4) then
      call dxz4th_op2(adj,add,model,data1,data2)
    else
      call dxz10th_op2(adj,add,model,data1,data2)
    endif
    stat=0
  endfunction

  ! Subroutines that are called by the functions
  ! d/dx subroutines
  subroutine dx4th_op2(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=-1.0/24.0
    c2= 9.0/8.0
    model=0.0
    do ix=3,nx-2
      do iz=3,nz-2
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz,ix+1,1)-data(iz,ix-2,1))+&
                                          c2*(data(iz,ix  ,1)-data(iz,ix-1,1))
      enddo
    enddo
  endsubroutine

  subroutine dx10th_op2(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=   35.0/294912.0
    c2= -405.0/229376.0
    c3=   567.0/40960.0
    c4=   -735.0/8192.0
    c5= 19845.0/16384.0
    model=0.0
    do ix=6,nx-5
      do iz=6,nz-5
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz,ix+4,1)-data(iz,ix-5,1))+&
                                          c2*(data(iz,ix+3,1)-data(iz,ix-4,1))+&
                                          c3*(data(iz,ix+2,1)-data(iz,ix-3,1))+&
                                          c4*(data(iz,ix+1,1)-data(iz,ix-2,1))+&
                                          c5*(data(iz,ix  ,1)-data(iz,ix-1,1))
      enddo
    enddo
  endsubroutine

  subroutine dx4th_op4(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=-1.0/24.0
    c2= 9.0/8.0
    model=0.0
    do ix=3,nx-2
      do iz=3,nz-2
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz,ix+2,1)-data(iz,ix-1,1))+&
                                          c2*(data(iz,ix+1,1)-data(iz,ix  ,1))
      enddo
    enddo
  endsubroutine

  subroutine dx10th_op4(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=   35.0/294912.0
    c2= -405.0/229376.0
    c3=   567.0/40960.0
    c4=   -735.0/8192.0
    c5= 19845.0/16384.0
    model=0.0
    do ix=6,nx-5
      do iz=6,nz-5
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz,ix+5,1)-data(iz,ix-4,1))+&
                                          c2*(data(iz,ix+4,1)-data(iz,ix-3,1))+&
                                          c3*(data(iz,ix+3,1)-data(iz,ix-2,1))+&
                                          c4*(data(iz,ix+2,1)-data(iz,ix-1,1))+&
                                          c5*(data(iz,ix+1,1)-data(iz,ix  ,1))
      enddo
    enddo
  endsubroutine

  ! d/dz subroutines
  subroutine dz4th_op2(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=-1.0/24.0
    c2= 9.0/8.0
    model=0.0
    do ix=3,nx-2
      do iz=3,nz-2
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz+1,ix,1)-data(iz-2,ix,1))+&
                                          c2*(data(iz  ,ix,1)-data(iz-1,ix,1))
      enddo
    enddo
  endsubroutine

  subroutine dz10th_op2(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=   35.0/294912.0
    c2= -405.0/229376.0
    c3=   567.0/40960.0
    c4=   -735.0/8192.0
    c5= 19845.0/16384.0
    model=0.0
    do ix=6,nx-5
      do iz=6,nz-5
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz+4,ix,1)-data(iz-5,ix,1))+&
                                          c2*(data(iz+3,ix,1)-data(iz-4,ix,1))+&
                                          c3*(data(iz+2,ix,1)-data(iz-3,ix,1))+&
                                          c4*(data(iz+1,ix,1)-data(iz-2,ix,1))+&
                                          c5*(data(iz  ,ix,1)-data(iz-1,ix,1))
      enddo
    enddo
  endsubroutine

  subroutine dz4th_op4(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=-1.0/24.0
    c2= 9.0/8.0
    model=0.0
    do ix=3,nx-2
      do iz=3,nz-2
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz+2,ix,1)-data(iz-1,ix,1))+&
                                          c2*(data(iz+1,ix,1)-data(iz  ,ix,1))
      enddo
    enddo
  endsubroutine

  subroutine dz10th_op4(adj,add,model,data)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data

    c1=   35.0/294912.0
    c2= -405.0/229376.0
    c3=   567.0/40960.0
    c4=   -735.0/8192.0
    c5= 19845.0/16384.0
    model=0.0
    do ix=6,nx-5
      do iz=6,nz-5
        model(iz,ix,1) = model(iz,ix,1) + c1*(data(iz+5,ix,1)-data(iz-4,ix,1))+&
                                          c2*(data(iz+4,ix,1)-data(iz-3,ix,1))+&
                                          c3*(data(iz+3,ix,1)-data(iz-2,ix,1))+&
                                          c4*(data(iz+2,ix,1)-data(iz-1,ix,1))+&
                                          c5*(data(iz+1,ix,1)-data(iz  ,ix,1))
      enddo
    enddo
  endsubroutine

  ! mixed derivative (shear stress) subroutines
  subroutine dxz4th_op2(adj,add,model,data1,data2)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data1,data2

    c1=-1.0/24.0
    c2= 9.0/8.0
    model=0.0
    do ix=3,nx-2
      do iz=3,nz-2
        !if(iz.lt.(nz-2))then
          model(iz,ix,1) = model(iz,ix,1) + c1*(data1(iz+1,ix,1)-data1(iz-2,ix,1))+&
                                            c2*(data1(iz  ,ix,1)-data1(iz-1,ix,1))+&
                                            c1*(data2(iz,ix+1,1)-data2(iz,ix-2,1))+&
                                            c2*(data2(iz,ix  ,1)-data2(iz,ix-1,1))
        !else
        !  model(iz,ix) = model(iz,ix) +     data1(iz,ix) - data1(iz-1,ix) + &
        !                                c1*(data2(iz+2,ix)-data2(iz-2,ix))+&
        !                                c2*(data2(iz+1,ix)-data2(iz-1,ix))
          !if(ix.eq.3) then
          !  model(iz,3) = model(iz,3) + data1(iz,3) - data1(iz-1,3) + &
          !                              data2(iz,3) - data2(iz,2)
          !endif
        !endif
      enddo
    enddo
  endsubroutine

  subroutine dxz10th_op2(adj,add,model,data1,data2)
    logical,optional            :: add,adj
    real,dimension(nz,nx,1)     :: model,data1, data2

    c1=   35.0/294912.0
    c2= -405.0/229376.0
    c3=   567.0/40960.0
    c4=   -735.0/8192.0
    c5= 19845.0/16384.0
    model=0.0
    do ix=6,nx-5
      do iz=6,nz-5
        model(iz,ix,1) = model(iz,ix,1) + c1*(data1(iz+4,ix,1)-data1(iz-5,ix,1))+&
                                          c2*(data1(iz+3,ix,1)-data1(iz-4,ix,1))+&
                                          c3*(data1(iz+2,ix,1)-data1(iz-3,ix,1))+&
                                          c4*(data1(iz+1,ix,1)-data1(iz-2,ix,1))+&
                                          c5*(data1(iz  ,ix,1)-data1(iz-1,ix,1))+&
                                          c1*(data2(iz,ix+4,1)-data2(iz,ix-5,1))+&
                                          c2*(data2(iz,ix+3,1)-data2(iz,ix-4,1))+&
                                          c3*(data2(iz,ix+2,1)-data2(iz,ix-3,1))+&
                                          c4*(data2(iz,ix+1,1)-data2(iz,ix-2,1))+&
                                          c5*(data2(iz,ix  ,1)-data2(iz,ix-1,1))
      enddo
    enddo
  endsubroutine

endmodule
