! Program created by Gustavo Catao Alves at
! the Stanford Exploration Project
! May, 6th, 2014

! seismogram module
module seis_mod

  use sep

  implicit none
  integer, private :: nz,nx,abc
  real, private    :: dx
  real, dimension(:,:), pointer, private :: seisp, seiss

  contains

  ! Initialization of all variables needed by this subroutine
  subroutine seis_init(nz_in,nx_in, dx_in, abc_in, seisp_in, seiss_in)
    integer,intent(in)  :: nz_in,nx_in, abc_in
    real,intent(in)     :: dx_in
    real, dimension(:,:), target :: seisp_in, seiss_in
    nz = nz_in
    nx = nx_in
    dx = dx_in
    abc = abc_in
    seisp => seisp_in
    seiss => seiss_in

  end subroutine

  ! Function that is called by elastic_mod
  function seis_op(countseis,Vx,Vz) result(stat)
    integer,intent(in)  :: countseis
    real,dimension(:)   :: Vx, Vz
    integer             :: stat
    call seis_op2(countseis,Vx,Vz)
    stat=0
  end function

  subroutine seis_op2(countseis,Vx,Vz)
    integer,intent(in)      :: countseis
    real,dimension(nz,nx,1) :: Vx, Vz
    integer                 :: ix
    real                    :: dxVx,dxVz,dzVx,dzVz,aux

    aux = 1.0/(2.0*dx)
    do ix=1,nx-2*abc
      dxVx = Vx(abc,ix+abc+1,1)-Vx(abc,ix+abc-1,1)
      dzVx = Vx(abc+1,ix+abc,1)-Vx(abc-1,ix+abc,1)
      dxVz = Vz(abc,ix+abc+1,1)-Vz(abc,ix+abc-1,1)
      dzVz = Vz(abc+1,ix+abc,1)-Vz(abc-1,ix+abc,1)

      ! Divergence of the velocity field, P seismogram
      seisp(countseis,ix) = (dxVx + dzVz)*aux
      ! Curl of the velocity field, S seismogram
      seiss(countseis,ix) = (dzVx - dxVz)*aux
    enddo

  endsubroutine

endmodule

